#include <DMXSerial.h>
#include <TimerOne.h>

/*
  DMX/Random-Ansteuerung für analoge Scopes

  DMX-Modus mit gesteckter Brücke: Änderungen auf Adresse 90 lösen einen Blitz aus
  Random-Modus ohne Brücke: Pseudo-Random Blitze für Sylvester mit max 2sec Blitzabstand

  V1.0 20200105 First Release
  V1.1 20210106 Auto-Strobomode
*/

const int STRBCTRL =  2;      // Digital Pin 2
const int MODESW =  3;      // Digital Pin 3 ist umschalter DMX oder Random

long randNumber;

int   iMode = 0;    // 0: DMX 1: Strbo
int   iDMXdata = 0;
int   iDMXdata_old = 0;
int   iPhase = 0;
long  lTime = 0;

// the setup function runs once when you press reset or power the board
void setup() {
  DMXSerial.init(DMXReceiver);
  // initialize digital pins
  pinMode(MODESW, INPUT_PULLUP);
  pinMode(LED_BUILTIN, OUTPUT);
  pinMode(STRBCTRL, OUTPUT);
  digitalWrite(STRBCTRL, HIGH);

  if (digitalRead(MODESW)== HIGH) {
    iMode = 1; 
  }

    Timer1.attachInterrupt(timer1_isr);
    Timer1.initialize(100000);
    
}

void trigger_strobo() {
    digitalWrite(LED_BUILTIN, HIGH);  // turn the LED on (HIGH is the voltage level)
    digitalWrite(STRBCTRL, LOW);      // Puls auf Trigger-Ausgang
    delay(11);                       // wait 
    digitalWrite(LED_BUILTIN, LOW);   // turn the LED off by making the voltage LOW
    digitalWrite(STRBCTRL, HIGH);  
}

void timer1_isr(void) {
    if (iDMXdata > 10){
      if (iPhase == 1){
            digitalWrite(LED_BUILTIN, HIGH);  // turn the LED on (HIGH is the voltage level)
            digitalWrite(STRBCTRL, LOW);      // Puls auf Trigger-Ausgang
             Timer1.initialize(10500);
             iPhase = 0;
      }
      else {
         digitalWrite(LED_BUILTIN, LOW);   // turn the LED off by making the voltage LOW
         digitalWrite(STRBCTRL, HIGH); 
         lTime =  (long)(iDMXdata-10);
         lTime *= 1000;
         lTime += 10000;
         Timer1.initialize(lTime);
         iPhase = 1;
      }
    }
}

// the loop function runs over and over again forever
void loop() {

  if ( iMode==1) {

    trigger_strobo();
    randNumber = random(11, 2000);
    delay(randNumber);                       // wait for a second
  }
  else {
    iDMXdata = DMXSerial.read(90);

    // Beim Wert 0 (Dunkel) nicht flashen
    if (iDMXdata == 0) {
      iDMXdata_old = 0;
    }
    // Beim Wert >10 autostrobe, dann nicht eventtriggern
    if (iDMXdata > 10)
    {
      // wenn neue Daten, dann Frequenz neu setzen
      if (iDMXdata != iDMXdata_old){
        iPhase = 1;
        Timer1.initialize(14000);
        iDMXdata_old = iDMXdata;
      }
    }
    else {
      iPhase = 0;
          digitalWrite(LED_BUILTIN, LOW);   // turn the LED off by making the voltage LOW
        digitalWrite(STRBCTRL, HIGH);  
    }
    
    
    // Bei Änderung Blitz auslösen
    if (iDMXdata != iDMXdata_old){
          iDMXdata_old = iDMXdata;
          trigger_strobo();
    }
    
     
 
  }
  
  
}
